# Copyright (C) 2007-2010 www.stani.be
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/
#
# Follows PEP8


import os
import shlex
import sys
import _winreg

PYTHONW = os.path.join(sys.prefix, 'pythonw.exe')
PY = 'Python.File'
PYW = 'Python.NoConFile'


def get_winreg_file_type(extension):
    """Convert a file extension in a Windows registery file type.

    :param extension: file extension
    :type extension: str
    :returns: file type
    :rtype: string

    > get_winreg_file_type('.py')
    'Python.File'
    > get_winreg_file_type('.pyw')
    'Python.NoConFile'
    """
    return _winreg.QueryValue(_winreg.HKEY_CLASSES_ROOT, extension)


#---register
def is_label_registered(label, extension):
    """Checks if a label registered to a certain extension.

    :param label: label
    :type label: str
    :param extension: file extension
    :type extension: str
    :returns: if registered or not
    :rtype: bool
    """
    try:
        filetype = get_winreg_file_type(extension)
    except WindowsError:
        return False
    key = '%s\\shell\\%s' % (filetype, label)
    try:
        _winreg.OpenKey(_winreg.HKEY_CLASSES_ROOT, key)
        return True
    except WindowsError:
        return False


def are_labels_registered(labels, extension):
    """Checks if multiple labels registered to a certain extension.

    :param labels: labels
    :type label: list of str
    :param extension: file extension
    :type extension: str
    :returns: if registered or not
    :rtype: bool
    """
    for label in labels:
        if is_label_registered(label, extension):
            return True
    return False


def which_registered(labels, extensions):
    """Checks which extensions are registered with a given list of labels.

    :param labels: labels
    :type label: list of str
    :param extensions: file extensions
    :type extensions: str
    :returns: registered extensions
    :rtype: list of str
    """
    return [ext for ext in extensions if are_labels_registered(labels, ext)]


def first_arg_ends_with(cmdline, extensions):
    """This will split the ``cmdline`` and check if the first argument
    ends with a certain extension.

    Helper function for func:`register_extension`.

    >>> first_arg_ends_with('"app.PY" --associate', ['.py','.pyw'])
    True
    >>> first_arg_ends_with('"app.PY" --associate', ['.pyw'])
    False
    """
    first_arg = shlex.split(cmdline.encode('ascii', 'ignore'))[0]
    return os.path.splitext(first_arg)[-1].lower() in extensions


def register(label, action, filetype='Python.File', suffix='"%1"'):
    """Register action with a label in the windows explorer context menu.

    :param label: label describing the action
    :type label: str
    :param action: command line for the action
    :type action: str
    :param filetype: file type in the Windows registry
    :type filetype: str
    :param suffix: suffix parameters for the action (command line)
    :type suffix: str
    :returns: ``True`` if successfull, ``False`` otherwise
    :rtype: bool
    """
    try:
        k = '%s\\shell\\%s' % (filetype, label)
        key = _winreg.CreateKey(_winreg.HKEY_CLASSES_ROOT, k)
    except:
        pass
    try:
        command = '%s %s' % (action, suffix)
        _winreg.SetValue(key, "command", _winreg.REG_SZ, command)
        return key
    except:
        return False


def register_extension(actions, extension, suffix='"%1"'):
    """Registering actions with an extension in windows explorer context menu.

    :param actions: actions by label
    :type actions: dict
    :param extension: extension of the file type
    :type extension: str
    :param suffix: suffix parameters for the action (command line)
    :type suffix: str
    :returns: ``True`` if successfull, ``False`` otherwise
    :rtype: bool

    If the registering of one action is unsuccesfull it will return False
    immediately without trying to register other actions.
    """
    try:
        filetype = get_winreg_file_type(extension)
    except:
        return False
    result = []
    for label, action in actions.items():
        if first_arg_ends_with(action, ('.py', '.pyw')):
            action = '"%s" %s' % (PYTHONW, action)
        if not register(
            label=label,
            action=action,
            filetype=filetype,
            suffix=suffix,
        ):
            return False
    return True


def register_extensions(actions, extensions, folder=False, suffix='"%1"'):
    """Registering actions with extensions in windows explorer context menu.

    :param actions: actions by label
    :type actions: dict
    :param extensions: extensions of the file types
    :type extensions: str
    :param folder: if needs to be registered with folders
    :type folder: bool
    :param suffix: suffix parameters for the action (command line)
    :type suffix: str
    :returns: list of successfully registered actions
    :rtype: list of str
    """
    result = []
    if folder:
        for label, action in actions.items():
            if register(label, action, 'Folder', suffix):
                result.append('folder')
    for extension in extensions:
        if register_extension(actions, extension, suffix):
            result.append(extension)
    return result


def register_py(label, action, suffix='"%1"'):
    """Register action for python (*.py, *.pyw) in windows
    explorer contextmenu."""
    keyw = {
        'label': label,
        'action': '"%s" "%s"' % (PYTHONW, action),
        'suffix': suffix}
    py, pyw = keys\
            = (register(filetype=PY, **keyw), register(filetype=PYW, **keyw))
    if py and pyw:
        return keys
    else:
        return False

#---deregister


def deregister(label, filetype='Python.File'):
    """Deregister label from a certain file type

    :param label: label describing the action
    :type label: str
    :param filetype: file type in the Windows registry
    :type filetype: str
    :returns: ``True`` if successfull, ``False`` otherwise
    :rtype: bool
    """
    try:
        key = '%s\\shell\\%s' % (filetype, label)
        _winreg.DeleteKey(_winreg.HKEY_CLASSES_ROOT, key + '\\command')
    except  Exception, message:
        pass
    try:
        _winreg.DeleteKey(_winreg.HKEY_CLASSES_ROOT, key)
        return True
    except  Exception, message:
        return False


def deregister_extension(labels, extension):
    """Deregister labels from a certain file extension

    :param labels: labels describing the actions
    :type labels: str
    :param extension: extension of the file type
    :type extension: str
    :returns: ``True`` if successfull for all labels, ``False`` otherwise
    :rtype: bool
    """
    try:
        filetype = get_winreg_file_type(extension)
    except:
        return False
    ok = True
    for label in labels:
        if not deregister(label, filetype):
            ok = False
    return ok


def deregister_extensions(labels, extensions, folder=True):
    """Deregister labels from certain file extensions

    :param labels: labels describing the actions
    :type labels: str
    :param extensions: extensions of the file types
    :type extensions: str
    :param folder: if needs to be registered with folders
    :type folder: bool
    :returns: ``True`` if successfull for all labels, ``False`` otherwise
    :rtype: bool
    :returns: list of successfully deregistered actions
    :rtype: list of str
    """
    result = []
    for label in labels:
        if folder:
            if deregister(label, 'Folder'):
                result.append('folder')
    for extension in extensions:
        if deregister_extension(labels, extension):
            result.append(extension)
    return result


def deregister_py(label):
    """Deregister action for python (*.py, *.pyw) in windows
    explorer contextmenu."""
    deregister(label=label, filetype=PY)
    deregister(label=label, filetype=PYW)
